@@ -52,6 +52,26 @@ module Puppet
         :owner    => "service",
         :group    => "service",
         :desc     => "The directory in which to store log files",
+    },
+    :log_level => {
+      :default    => 'notice',
+      :type       => :enum,
+      :values     => ["debug","info","notice","warning","err","alert","emerg","crit"],
+      :desc       => "Default logging level",
+    },
+    :disable_warnings => {
+      :default => [],
+      :type    => :array,
+      :desc    => "A list of warning types to disable. Currently the only warning type that can be
+        disabled are deprecations, but more warning types may be added later.",
+      :hook      => proc do |value|
+        values = munge(value)
+        valid   = %w[deprecations]
+        invalid = values - (values & valid)
+        if not invalid.empty?
+          raise ArgumentError, "Cannot disable unrecognized warning types #{invalid.inspect}. Valid values are #{valid.inspect}."
+        end
+      end
     }
   )
 
@@ -196,6 +216,17 @@ module Puppet
           is used to find modules and much more.  For servers (i.e., `puppet master`)
           this provides the default environment for nodes we know nothing about."
     },
+    :environmentpath => {
+      :default => "",
+      :desc    => "A search path for directory environments, as a list of directories
+        separated by the system path separator character. (The POSIX path separator
+        is ':', and the Windows path separator is ';'.)
+
+        This setting must have a value set to enable **directory environments.** The
+        recommended value is `$confdir/environments`. For more details, see
+        http://docs.puppetlabs.com/puppet/latest/reference/environments.html",
+      :type    => :path,
+    },
     :diff_args => {
         :default  => default_diffargs,
         :desc     => "Which arguments to pass to the diff command when printing differences between
@@ -251,7 +282,7 @@ module Puppet
       :type       => :terminus,
       :default    => nil,
       :desc       => "How to store cached nodes.
-      Valid values are (none), 'json', 'yaml' or write only yaml ('write_only_yaml').
+      Valid values are (none), 'json', 'msgpack', 'yaml' or write only yaml ('write_only_yaml').
       The master application defaults to 'write_only_yaml', all others to none.",
     },
     :data_binding_terminus => {
@@ -266,8 +297,9 @@ module Puppet
     },
     :binder => {
       :default => false,
-      :desc    => "Turns the binding system on or off. This includes hiera-2 and data in modules.  The binding system aggregates data from
-      modules and other locations and makes them available for lookup.  The binding system is experimental and any or all of it may change.",
+      :desc    => "Turns the binding system on or off. This includes bindings in modules.
+        The binding system aggregates data from modules and other locations and makes them available for lookup.
+        The binding system is experimental and any or all of it may change.",
       :type    => :boolean,
     },
     :binder_config => {
@@ -286,7 +318,7 @@ module Puppet
     :catalog_cache_terminus => {
       :type       => :terminus,
       :default    => nil,
-      :desc       => "How to store cached catalogs. Valid values are 'json' and 'yaml'. The agent application defaults to 'json'."
+      :desc       => "How to store cached catalogs. Valid values are 'json', 'msgpack' and 'yaml'. The agent application defaults to 'json'."
     },
     :facts_terminus => {
       :default => 'facter',
@@ -337,6 +369,12 @@ module Puppet
       configuration files.  This timeout determines how quickly Puppet checks whether
       a file (such as manifests or templates) has changed on disk. #{AS_DURATION}",
     },
+    :environment_timeout => {
+      :default    => "3m",
+      :type       => :ttl,
+      :desc       => "The time to live for a cached environment. The time is either given #{AS_DURATION}, or
+      the word 'unlimited' which causes the environment to be cached until the master is restarted."
+    },
     :queue_type => {
       :default    => "stomp",
       :desc       => "Which type of queue to use for asynchronous processing.",
@@ -358,11 +396,11 @@ module Puppet
         :hook     => proc do |value|
           if value
             # This reconfigures the termini for Node, Facts, and Catalog
-            Puppet.settings[:storeconfigs] = true
+            Puppet.settings.override_default(:storeconfigs, true)
 
             # But then we modify the configuration
             Puppet::Resource::Catalog.indirection.cache_class = :queue
-            Puppet.settings[:catalog_cache_terminus] = :queue
+            Puppet.settings.override_default(:catalog_cache_terminus, :queue)
           else
             raise "Cannot disable asynchronous storeconfigs in a running process"
           end
@@ -377,7 +415,7 @@ module Puppet
     `active_record` backend, but will disable external tools that search the storeconfigs database.
     Thinning catalogs is generally unnecessary when using PuppetDB to store catalogs.",
       :hook     => proc do |value|
-        Puppet.settings[:storeconfigs] = true if value
+        Puppet.settings.override_default(:storeconfigs, true) if value
         end
       },
     :config_version => {
@@ -385,7 +423,12 @@ module Puppet
       :desc       => "How to determine the configuration version.  By default, it will be the
       time that the configuration is parsed, but you can provide a shell script to override how the
       version is determined.  The output of this script will be added to every log message in the
-      reports, allowing you to correlate changes on your hosts to the source version on the server.",
+      reports, allowing you to correlate changes on your hosts to the source version on the server.
+
+      Setting a global value for config_version in puppet.conf is deprecated. Please set a
+      per-environment value in environment.conf instead. For more info, see
+      http://docs.puppetlabs.com/puppet/latest/reference/environments.html",
+      :deprecated => :allowed_on_commandline,
     },
     :zlib => {
         :default  => true,
@@ -421,11 +464,16 @@ module Puppet
       :type    => :boolean,
       :desc    => "Stores trusted node data in a hash called $trusted.
         When true also prevents $trusted from being overridden in any scope.",
+    },
+    :immutable_node_data => {
+      :default => '$trusted_node_data',
+      :type    => :boolean,
+      :desc    => "When true, also prevents $trusted and $facts from being overridden in any scope",
     }
   )
   Puppet.define_settings(:module_tool,
     :module_repository  => {
-      :default  => 'https://forge.puppetlabs.com',
+      :default  => 'https://forgeapi.puppetlabs.com',
       :desc     => "The module repository",
     },
     :module_working_dir => {
@@ -533,7 +581,9 @@ EOT
     :certdir => {
       :default => "$ssldir/certs",
       :type   => :directory,
+      :mode => 0755,
       :owner => "service",
+      :group => "service",
       :desc => "The certificate directory."
     },
     :ssldir => {
@@ -541,18 +591,23 @@ EOT
       :type   => :directory,
       :mode => 0771,
       :owner => "service",
+      :group => "service",
       :desc => "Where SSL certificates are kept."
     },
     :publickeydir => {
       :default => "$ssldir/public_keys",
       :type   => :directory,
+      :mode => 0755,
       :owner => "service",
+      :group => "service",
       :desc => "The public key directory."
     },
     :requestdir => {
       :default => "$ssldir/certificate_requests",
       :type => :directory,
+      :mode => 0755,
       :owner => "service",
+      :group => "service",
       :desc => "Where host certificate requests are stored."
     },
     :privatekeydir => {
@@ -560,6 +615,7 @@ EOT
       :type   => :directory,
       :mode => 0750,
       :owner => "service",
+      :group => "service",
       :desc => "The private key directory."
     },
     :privatedir => {
@@ -567,6 +623,7 @@ EOT
       :type   => :directory,
       :mode => 0750,
       :owner => "service",
+      :group => "service",
       :desc => "Where the client stores private certificate information."
     },
     :passfile => {
@@ -574,6 +631,7 @@ EOT
       :type   => :file,
       :mode => 0640,
       :owner => "service",
+      :group => "service",
       :desc => "Where puppet agent stores the password for its private key.
         Generally unused."
     },
@@ -582,6 +640,7 @@ EOT
       :type   => :file,
       :mode => 0644,
       :owner => "service",
+      :group => "service",
       :desc => "Where individual hosts store and look for their certificate requests."
     },
     :hostcert => {
@@ -589,13 +648,15 @@ EOT
       :type   => :file,
       :mode => 0644,
       :owner => "service",
+      :group => "service",
       :desc => "Where individual hosts store and look for their certificates."
     },
     :hostprivkey => {
       :default => "$privatekeydir/$certname.pem",
       :type   => :file,
-      :mode => 0600,
+      :mode => 0640,
       :owner => "service",
+      :group => "service",
       :desc => "Where individual hosts store and look for their private key."
     },
     :hostpubkey => {
@@ -603,6 +664,7 @@ EOT
       :type   => :file,
       :mode => 0644,
       :owner => "service",
+      :group => "service",
       :desc => "Where individual hosts store and look for their public key."
     },
     :localcacert => {
@@ -610,14 +672,16 @@ EOT
       :type   => :file,
       :mode => 0644,
       :owner => "service",
+      :group => "service",
       :desc => "Where each client stores the CA certificate."
     },
     :ssl_client_ca_auth => {
       :type  => :file,
       :mode  => 0644,
       :owner => "service",
+      :group => "service",
       :desc  => "Certificate authorities who issue server certificates.  SSL servers will not be
-        considered authentic unless they posses a certificate issued by an authority
+        considered authentic unless they possess a certificate issued by an authority
         listed in this file.  If this setting has no value then the Puppet master's CA
         certificate (localcacert) will be used."
     },
@@ -625,8 +689,9 @@ EOT
       :type  => :file,
       :mode  => 0644,
       :owner => "service",
+      :group => "service",
       :desc  => "Certificate authorities who issue client certificates.  SSL clients will not be
-        considered authentic unless they posses a certificate issued by an authority
+        considered authentic unless they possess a certificate issued by an authority
         listed in this file.  If this setting has no value then the Puppet master's CA
         certificate (localcacert) will be used."
     },
@@ -635,6 +700,7 @@ EOT
       :type   => :file,
       :mode => 0644,
       :owner => "service",
+      :group => "service",
       :desc => "Where the host's certificate revocation list can be found.
         This is distinct from the certificate authority's CRL."
     },
@@ -650,6 +716,13 @@ EOT
       :type     => :duration,
       :desc     => "The window of time leading up to a certificate's expiration that a notification
         will be logged. This applies to CA, master, and agent certificates. #{AS_DURATION}"
+    },
+    :digest_algorithm => {
+        :default  => 'md5',
+        :type     => :enum,
+        :values => ["md5", "sha256"],
+        :desc     => 'Which digest algorithm to use for file resources and the filebucket.
+                      Valid values are md5, sha256. Default is md5.',
     }
   )
 
@@ -664,7 +737,7 @@ EOT
       :type => :directory,
       :owner => "service",
       :group => "service",
-      :mode => 0770,
+      :mode => 0755,
       :desc => "The root directory for the certificate authority."
     },
     :cacert => {
@@ -672,7 +745,7 @@ EOT
       :type => :file,
       :owner => "service",
       :group => "service",
-      :mode => 0660,
+      :mode => 0644,
       :desc => "The CA certificate."
     },
     :cakey => {
@@ -680,7 +753,7 @@ EOT
       :type => :file,
       :owner => "service",
       :group => "service",
-      :mode => 0660,
+      :mode => 0640,
       :desc => "The CA private key."
     },
     :capub => {
@@ -688,6 +761,7 @@ EOT
       :type => :file,
       :owner => "service",
       :group => "service",
+      :mode => 0644,
       :desc => "The CA public key."
     },
     :cacrl => {
@@ -695,8 +769,7 @@ EOT
       :type => :file,
       :owner => "service",
       :group => "service",
-      :mode => 0664,
-
+      :mode => 0644,
       :desc => "The certificate revocation list (CRL) for the CA. Will be used if present but otherwise ignored.",
     },
     :caprivatedir => {
@@ -704,7 +777,7 @@ EOT
       :type => :directory,
       :owner => "service",
       :group => "service",
-      :mode => 0770,
+      :mode => 0750,
       :desc => "Where the CA stores private certificate information."
     },
     :csrdir => {
@@ -712,6 +785,7 @@ EOT
       :type => :directory,
       :owner => "service",
       :group => "service",
+      :mode  => 0755,
       :desc => "Where the CA stores certificate requests"
     },
     :signeddir => {
@@ -719,7 +793,7 @@ EOT
       :type => :directory,
       :owner => "service",
       :group => "service",
-      :mode => 0770,
+      :mode => 0755,
       :desc => "Where the CA stores signed certificates."
     },
     :capass => {
@@ -727,7 +801,7 @@ EOT
       :type => :file,
       :owner => "service",
       :group => "service",
-      :mode => 0660,
+      :mode => 0640,
       :desc => "Where the CA stores the password for the private key."
     },
     :serial => {
@@ -775,7 +849,7 @@ EOT
     :ca_ttl => {
       :default    => "5y",
       :type       => :duration,
-      :desc       => "The default TTL for new certificates. If this setting is set, ca_days is ignored.
+      :desc       => "The default TTL for new certificates.
       #{AS_DURATION}"
     },
     :req_bits => {
@@ -836,12 +910,24 @@ EOT
     :manifestdir => {
       :default    => "$confdir/manifests",
       :type       => :directory,
-      :desc       => "Where puppet master looks for its manifests.",
+      :desc       => "Used to build the default value of the `manifest` setting. Has no other purpose.
+
+        This setting is deprecated.",
+      :deprecated => :completely,
     },
     :manifest => {
       :default    => "$manifestdir/site.pp",
-      :type       => :file,
-      :desc       => "The entry-point manifest for puppet master.",
+      :type       => :file_or_directory,
+      :desc       => "The entry-point manifest for puppet master. This can be one file
+        or a directory of manifests to be evaluated in alphabetical order. Puppet manages
+        this path as a directory if one exists or if the path ends with a / or \\.
+
+        Setting a global value for `manifest` in puppet.conf is deprecated. Please use
+        directory environments instead. If you need to use something other than the
+        environment's `manifests` directory as the main manifest, you can set
+        `manifest` in environment.conf. For more info, see
+        http://docs.puppetlabs.com/puppet/latest/reference/environments.html",
+      :deprecated => :allowed_on_commandline,
     },
     :code => {
       :default    => "",
@@ -902,12 +988,34 @@ EOT
       :type       => :boolean,
       :desc       => "Whether the master should function as a certificate authority.",
     },
-    :modulepath => {
+    :basemodulepath => {
       :default => "$confdir/modules#{File::PATH_SEPARATOR}/usr/local/share/puppet/modules",
       :type => :path,
+      :desc => "The search path for **global** modules. Should be specified as a
+        list of directories separated by the system path separator character. (The
+        POSIX path separator is ':', and the Windows path separator is ';'.)
+
+        If you are using directory environments, these are the modules that will
+        be used by _all_ environments. Note that the `modules` directory of the active
+        environment will have priority over any global directories. For more info, see
+        http://docs.puppetlabs.com/puppet/latest/reference/environments.html
+
+        This setting also provides the default value for the deprecated `modulepath`
+        setting, which is used when directory environments are disabled.",
+    },
+    :modulepath => {
+      :default => "$basemodulepath",
+      :type => :path,
       :desc => "The search path for modules, as a list of directories separated by the system
         path separator character. (The POSIX path separator is ':', and the
-        Windows path separator is ';'.)",
+        Windows path separator is ';'.)
+
+        Setting a global value for `modulepath` in puppet.conf is deprecated. Please use
+        directory environments instead. If you need to use something other than the
+        default modulepath of `<ACTIVE ENVIRONMENT'S MODULES DIR>:$basemodulepath`,
+        you can set `modulepath` in environment.conf. For more info, see
+        http://docs.puppetlabs.com/puppet/latest/reference/environments.html",
+      :deprecated => :allowed_on_commandline,
     },
     :ssl_client_header => {
       :default    => "HTTP_X_CLIENT_DN",
@@ -1189,7 +1297,7 @@ EOT
       :hook => proc { |value|
         if value
           Puppet.deprecation_warning "Setting 'catalog_format' is deprecated; use 'preferred_serialization_format' instead."
-          Puppet.settings[:preferred_serialization_format] = value
+          Puppet.settings.override_default(:preferred_serialization_format, value)
         end
       }
     },
@@ -1465,7 +1573,11 @@ EOT
         separator is ':', and the Windows path separator is ';'.)",
 
       :call_hook => :on_initialize_and_write, # Call our hook with the default value, so we always get the value added to facter.
-      :hook => proc { |value| Facter.search(value) if Facter.respond_to?(:search) }}
+      :hook => proc do |value|
+        paths = value.split(File::PATH_SEPARATOR)
+        Facter.search(*paths)
+      end
+    }
   )
 
 
@@ -1707,7 +1819,7 @@ EOT
         if value
           if not Puppet.settings[:async_storeconfigs]
             Puppet::Resource::Catalog.indirection.cache_class = :store_configs
-            Puppet.settings[:catalog_cache_terminus] = :store_configs
+            Puppet.settings.override_default(:catalog_cache_terminus, :store_configs)
           end
           Puppet::Node::Facts.indirection.cache_class = :store_configs
 
@@ -1729,7 +1841,10 @@ EOT
         :default  => "$vardir/templates",
         :type     => :directory,
         :desc     => "Where Puppet looks for template files.  Can be a list of colon-separated
-          directories.",
+          directories.
+
+          This setting is deprecated. Please put your templates in modules instead.",
+        :deprecated => :completely,
     },
 
     :allow_variables_with_dashes => {
@@ -1750,19 +1865,63 @@ EOT
       :default => "current",
       :desc => <<-'EOT'
         Selects the parser to use for parsing puppet manifests (in puppet DSL
-        language/'.pp' files). Available choices are `current` (the default),
+        language/'.pp' files). Available choices are `current` (the default)
         and `future`.
 
         The `curent` parser means that the released version of the parser should
         be used.
 
         The `future` parser is a "time travel to the future" allowing early
-        exposure to new language features. What these fatures are will vary from
+        exposure to new language features. What these features are will vary from
         release to release and they may be invididually configurable.
 
         Available Since Puppet 3.2.
       EOT
     },
+    :evaluator => {
+      :default => "future",
+      :hook => proc do |value|
+        if !['future', 'current'].include?(value)
+          raise "evaluator can only be set to 'future' or 'current', got '#{value}'"
+        end
+      end,
+      :desc => <<-'EOT'
+        Which evaluator to use when compiling Puppet manifests. Valid values
+        are `current` and `future` (the default).
+
+        **Note:** This setting is only used when `parser = future`. It allows
+        testers to turn off the `future` evaluator when doing detailed tests and
+        comparisons of the new compilation system.
+
+        Evaluation is the second stage of catalog compilation. After the parser
+        converts a manifest to a model of expressions, the evaluator processes
+        each expression. (For example, a resource declaration signals the
+        evaluator to add a resource to the catalog).
+
+        The `future` parser and evaluator are slated to become default in Puppet
+        4. Their purpose is to add new features and improve consistency
+        and reliability.
+
+        Available Since Puppet 3.5.
+      EOT
+    },
+   :biff => {
+     :default => false,
+     :type => :boolean,
+     :hook => proc do |value|
+       if Puppet.settings[:parser] != 'future'
+         Puppet.settings.override_default(:parser, 'future')
+       end
+       if Puppet.settings[:evaluator] != 'future'
+         Puppet.settings.override_default(:evaluator, 'future')
+       end
+     end,
+     :desc => <<-EOT
+       Turns on Biff the catalog builder, future parser, and future evaluator.
+       This is an experimental feature - and this setting may go away before
+       release of Pupet 3.6.
+     EOT
+   },
    :max_errors => {
      :default => 10,
      :desc => <<-'EOT'
@@ -1786,8 +1945,15 @@ EOT
       warnings in case multiple errors have been detected. A value of 0 is the
       same as value 1. The count is per manifest.
     EOT
+    },
+  :strict_variables => {
+    :default => false,
+    :type => :boolean,
+    :desc => <<-'EOT'
+      Makes the parser raise errors when referencing unknown variables. (This does not affect
+      referencing variables that are explicitly set to undef).
+    EOT
     }
-
   )
   define_settings(:puppetdoc,
     :document_all => {
