@@ -4,7 +4,9 @@ require 'puppet/provider/package'
 Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Package do
   desc "OpenBSD's form of `pkg_add` support."
 
-  commands :pkginfo => "pkg_info", :pkgadd => "pkg_add", :pkgdelete => "pkg_delete"
+  commands :pkginfo   => "pkg_info",
+           :pkgadd    => "pkg_add",
+           :pkgdelete => "pkg_delete"
 
   defaultfor :operatingsystem => :openbsd
   confine :operatingsystem => :openbsd
@@ -36,9 +38,11 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
             packages << new(hash)
             hash = {}
           else
-            # Print a warning on lines we can't match, but move
-            # on, since it should be non-fatal
-            warning("Failed to match line #{line}")
+            unless line =~ /Updating the pkgdb/
+              # Print a warning on lines we can't match, but move
+              # on, since it should be non-fatal
+              warning("Failed to match line #{line}")
+            end
           end
         }
       end
@@ -62,26 +66,40 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
       e_vars = {}
     end
 
-    output = Puppet::Util.withenv(e_vars) {pkginfo "-Q", resource[:name]}
-
-    if output.nil? or output.size == 0 or output =~ /Error from /
-      Puppet.debug("Failed to query for #{resource[:name]}")
-      return properties[:ensure]
+    if @resource[:flavor]
+      query = "#{@resource[:name]}--#{@resource[:flavor]}"
     else
-      output.chomp!
-      Puppet.debug("pkg_info -Q for #{resource[:name]}: #{output}")
+      query = @resource[:name]
     end
 
-    if output =~ /^#{resource[:name]}-(\d[^-]*)[-]?(\w*) \(installed\)$/
-      Puppet.debug("Package is already the latest available")
+    output = Puppet::Util.withenv(e_vars) {pkginfo "-Q", query}.split('\n')
+
+    if output.nil? or output.size == 0
+      debug "Failed to query for #{query}"
       return properties[:ensure]
     else
-      match = /^(.*)-(\d[^-]*)[-]?(\w*)$/.match(output)
-      Puppet.debug("Latest available for #{resource[:name]}: #{match[2]}")
-      if properties[:ensure] > match[2]
+      # Remove all fuzzy matches first.
+      output = output.select {|p| p =~ /^#{resource[:name]}-(\d[^-]*)[-]?(\w*)/}.join.chomp
+      debug "pkg_info -Q for #{query}: #{output}"
+    end
+
+    if output =~ /^#{resource[:name]}-(\d[^-]*)[-]?(\w*)(-#{resource[:flavor]})? \(installed\)$/
+      debug "Package is already the latest available"
+      return properties[:ensure]
+    else
+      # Flavors need a little nudge in the right direction
+      match = /(^#{resource[:name]})-(\d[^-]*)[-]?(\w*)(-#{resource[:flavor]})?/.match(output)
+      debug "Latest available for #{query}: #{match[2]}"
+      if properties[:ensure].to_sym == :absent
+        return match[2]
+      end
+
+      vcmp = properties[:ensure].split('.').map{|s|s.to_i} <=> match[2].split('.').map{|s|s.to_i}
+      if vcmp > 0
+        debug "ensure: #{properties[:ensure]}"
         # The locally installed package may actually be newer than what a mirror
         # has. Log it at debug, but ignore it otherwise.
-        Puppet.debug("Package #{resource[:name]} #{properties[:ensure]} newer then available #{match[2]}")
+        debug "Package #{resource[:name]} #{properties[:ensure]} newer then available #{match[2]}"
         return properties[:ensure]
       else
         return match[2]
@@ -95,7 +113,7 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
 
   def parse_pkgconf
     unless @resource[:source]
-      if Puppet::FileSystem::File.exist?("/etc/pkg.conf")
+      if Puppet::FileSystem.exist?("/etc/pkg.conf")
         File.open("/etc/pkg.conf", "rb").readlines.each do |line|
           if matchdata = line.match(/^installpath\s*=\s*(.+)\s*$/i)
             @resource[:source] = matchdata[1]
@@ -126,25 +144,31 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
 
     if @resource[:source][-1,1] == ::File::SEPARATOR
       e_vars = { 'PKG_PATH' => @resource[:source] }
-      full_name = [ @resource[:name], get_version || @resource[:ensure], @resource[:flavor] ].join('-').chomp('-').chomp('-')
+      # In case of a real update (i.e., the package already exists) then
+      # pkg_add(8) can handle the flavors. However, if we're actually
+      # installing with 'latest', we do need to handle the flavors.
+      # So we always need to handle flavors ourselves as to not break installs.
+      if latest and resource[:flavor]
+        full_name = "#{resource[:name]}--#{resource[:flavor]}"
+      elsif latest
+        # Don't depend on get_version for updates.
+        full_name = @resource[:name]
+      else
+        full_name = [ @resource[:name], get_version || @resource[:ensure], @resource[:flavor] ].join('-').chomp('-').chomp('-')
+      end
     else
       e_vars = {}
       full_name = @resource[:source]
     end
 
     cmd << install_options
-
-    # In case of a real update (i.e., the package already exists) then
-    # pkg_add(8) can handle the flavors. However, if we're actually
-    # installing with 'latest', we do need to handle the flavors.
-    # So we always need to handle flavors ourselves as to not break installs.
-    if latest and resource[:flavor]
-      full_name = "#{resource[:name]}--#{resource[:flavor]}"
-    end
-
     cmd << full_name
 
-    Puppet::Util.withenv(e_vars) { pkgadd cmd.flatten.compact.join(' ') }
+    if latest
+      cmd.unshift('-rz')
+    end
+
+    Puppet::Util.withenv(e_vars) { pkgadd cmd.flatten.compact }
   end
 
   def get_version
@@ -153,18 +177,10 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
       regex = /^(.*)-(\d[^-]*)[-]?(\w*)(.*)$/
       master_version = 0
       version = -1
-
       process.each_line do |line|
         if match = regex.match(line.split[0])
           # now we return the first version, unless ensure is latest
-          # also a flavor  needs to be taken into account as there
-          # may be a single package, which happens to be flavored.
           version = match.captures[1]
-          flavor = match.captures[2]
-          if !flavor.empty?
-            version = "#{version}-#{flavor}"
-          end
-
           return version unless @resource[:ensure] == "latest"
 
           master_version = version unless master_version > version
@@ -196,32 +212,8 @@ Puppet::Type.type(:package).provide :openbsd, :parent => Puppet::Provider::Packa
     join_options(resource[:uninstall_options])
   end
 
-  # Turns a array of options into flags to be passed to pkg_add(8) and
-  # pkg_delete(8). The options can be passed as a string or hash. Note
-  # that passing a hash should only be used in case -Dfoo=bar must be passed,
-  # which can be accomplished with:
-  #     install_options => [ { '-Dfoo' => 'bar' } ]
-  # Regular flags like '-L' must be passed as a string.
-  # @param options [Array]
-  # @return Concatenated list of options
-  # @api private
-  def join_options(options)
-    return unless options
-
-    options.collect do |val|
-      case val
-      when Hash
-        val.keys.sort.collect do |k|
-          "#{k}=#{val[k]}"
-        end.join(' ')
-      else
-        val
-      end
-    end
-  end
-
   def uninstall
-    pkgdelete uninstall_options.flatten.compact.join(' '), @resource[:name]
+    pkgdelete uninstall_options.flatten.compact, @resource[:name]
   end
 
   def purge
