@@ -80,6 +80,11 @@ Puppet::Type.type(:user).provide :useradd, :parent => Puppet::Provider::NameServ
     false
   end
 
+  def shell=(value)
+    check_valid_shell
+    set("shell", value)
+  end
+
   verify :gid, "GID must be an integer" do |value|
     value.is_a? Integer
   end
@@ -92,6 +97,7 @@ Puppet::Type.type(:user).provide :useradd, :parent => Puppet::Provider::NameServ
   has_features :system_users unless %w{HP-UX Solaris}.include? Facter.value(:operatingsystem)
 
   has_features :manages_passwords, :manages_password_age if Puppet.features.libshadow?
+  has_features :manages_shell
 
   def check_allow_dup
     # We have to manually check for duplicates when using libuser
@@ -108,6 +114,15 @@ Puppet::Type.type(:user).provide :useradd, :parent => Puppet::Provider::NameServ
     []
   end
 
+  def check_valid_shell
+    unless File.exists?(@resource.should(:shell))
+      raise(Puppet::Error, "Shell #{@resource.should(:shell)} must exist")
+    end
+    unless File.executable?(@resource.should(:shell).to_s)
+      raise(Puppet::Error, "Shell #{@resource.should(:shell)} must be executable")
+    end
+  end
+
   def check_manage_home
     cmd = []
     if @resource.managehome? and not @resource.forcelocal?
@@ -171,11 +186,7 @@ Puppet::Type.type(:user).provide :useradd, :parent => Puppet::Provider::NameServ
   end
 
   def deletecmd
-    if @resource.forcelocal?
-       cmd = [command(:localdelete)]
-    else
-       cmd = [command(:delete)]
-    end
+    cmd = [command(:delete)]
     cmd += @resource.managehome? ? ['-r'] : []
     cmd << @resource[:name]
   end
@@ -202,6 +213,9 @@ Puppet::Type.type(:user).provide :useradd, :parent => Puppet::Provider::NameServ
   end
 
   def create
+    if @resource[:shell]
+      check_valid_shell
+    end
      super
      if @resource.forcelocal? and self.groups?
        set(:groups, @resource[:groups])
