@@ -21,7 +21,9 @@ Puppet::Type.newtype(:resources) do
   newparam(:purge, :boolean => true, :parent => Puppet::Parameter::Boolean) do
     desc "Purge unmanaged resources.  This will delete any resource
       that is not specified in your configuration
-      and is not required by any specified resources."
+      and is not required by any specified resources.
+      Purging ssh_authorized_keys this way is deprecated; see the
+      purge_ssh_keys parameter of the user type for a better alternative."
 
     defaultto :false
 
@@ -37,7 +39,7 @@ Puppet::Type.newtype(:resources) do
 
   newparam(:unless_system_user) do
     desc "This keeps system users from being purged.  By default, it
-      does not purge users whose UIDs are less than or equal to 500, but you can specify
+      does not purge users whose UIDs are less than 500 or 1000, depending on operating system, but you can specify
       a different UID as the inclusive limit."
 
     newvalues(:true, :false, /^\d+$/)
@@ -47,11 +49,7 @@ Puppet::Type.newtype(:resources) do
       when /^\d+/
         Integer(value)
       when :true, true
-        if Facter.value('operatingsystem') != 'OpenBSD'
-          500
-        else
-          1000
-        end
+        @resource.system_users_max_uid
       when :false, false
         false
       when Integer; value
@@ -62,17 +60,35 @@ Puppet::Type.newtype(:resources) do
 
     defaultto {
       if @resource[:name] == "user"
-        if Facter.value('operatingsystem') != 'OpenBSD'
-          500
-        else
-          1000
-        end
+        @resource.system_users_max_uid
       else
         nil
       end
     }
   end
 
+  newparam(:unless_uid) do
+     desc "This keeps specific uids or ranges of uids from being purged when purge is true.
+       Accepts ranges, integers and (mixed) arrays of both."
+
+     munge do |value|
+       case value
+       when /^\d+/
+         [Integer(value)]
+       when Integer
+         [value]
+       when Range
+         [value]
+       when Array
+         value
+       when /^\[\d+/
+         value.split(',').collect{|x| x.include?('..') ? Integer(x.split('..')[0])..Integer(x.split('..')[1]) : Integer(x) }
+       else
+         raise ArgumentError, "Invalid value #{value.inspect}"
+       end
+     end
+   end
+
   def check(resource)
     @checkmethod ||= "#{self[:name]}_check"
     @hascheck ||= respond_to?(@checkmethod)
@@ -120,21 +136,37 @@ Puppet::Type.newtype(:resources) do
     @resource_type
   end
 
-  # Make sure we don't purge users below a certain uid, if the check
-  # is enabled.
+  # Make sure we don't purge users with specific uids
   def user_check(resource)
     return true unless self[:name] == "user"
     return true unless self[:unless_system_user]
-
     resource[:audit] = :uid
+    current_values = resource.retrieve_resource
+    current_uid = current_values[resource.property(:uid)]
+    unless_uids = self[:unless_uid]
 
     return false if system_users.include?(resource[:name])
 
-    current_values = resource.retrieve_resource
-    current_values[resource.property(:uid)] > self[:unless_system_user]
+    if unless_uids && unless_uids.length > 0
+      unless_uids.each do |unless_uid|
+        return false if unless_uid == current_uid
+        return false if unless_uid.respond_to?('include?') && unless_uid.include?(current_uid)
+      end
+    end
+
+    current_uid > self[:unless_system_user]
   end
 
   def system_users
     %w{root nobody bin noaccess daemon sys}
   end
+
+  def system_users_max_uid
+    case Facter.value(:osfamily)
+    when 'Debian', 'OpenBSD', 'FreeBSD'
+      999
+    else
+      499
+    end
+  end
 end
